<%inherit file="base/slide.html" />
<%block name="slide_classes">headerfooter</%block>
<%block name="content">
	<div class="header">
		<%block name="header">
			<div class="heading">${slide.var("heading", "")}</div>
		</%block>
	</div>
	<div class="body">
		<div class="content">
			${next.body()}
		</div>
	</div>
	<div class="footer">
		<%block name="footer">
			<div class="flex">
				<div class="flexchild left">
					%if (slide.var("toc_entry") is not None) and (len(slide.var("toc_entry").full_text) >= 1):
						${slide.var("toc_entry").full_text[0]}
					%endif
				</div>
				<div class="flexchild center">
					%if (slide.var("toc_entry") is not None) and (len(slide.var("toc_entry").full_text) >= 2):
						${slide.var("toc_entry").full_text[1]}
					%endif
				</div>
				<div class="flexchild right">${slide.var("current_slide_number")} / ${slide.var("total_slide_count")}</div>
			</div>
			<img src="${preuri}template/antonio/bar.svg" class="bar" />
		</%block>
	</div>
</%block>
