<%inherit file="template_slide.html" />
<%block name="header">
	<div class="heading">${slide.var("heading", "Table of Contents")}</div>
</%block>
<% toc = slide.var("partial_toc") %>
%if toc is None:
TOC not rendered yet.
%else:
%for (command, entry) in toc:
%if command.name == "NestingIncrease":
<ul class="bare">
%elif command.name == "NestingDecrease":
</ul>
%else:
<li>
%if len(entry.pages) > 0:
<a href="#slide${min(entry.pages)}">
%endif
${entry.full_number} ${entry.text}
%if len(entry.pages) > 0:
</a>
%endif
</li>
%endif
%endfor
%endif
