<%inherit file="base/slide.html" />
<%block name="slide_classes">final</%block>
<div class="body">
	<div class="final_flex">
		<div class="final_main">${slide.var("title", "Thank you!")}</div>
		%if slide.var("subtitle") != "":
			<div class="final_sub">${slide.var("subtitle", "Any questions?")}</div>
		%endif
	</div>
</div>
<div class="footer">
	<img src="${preuri}template/antonio/bar.svg" class="bar" />
</div>
